###Program to analyze two genes, but more genes can be added ####
library(fda)
library(MASS)
library(Matrix)
library(SKAT)

setwd("C:/NICHD/Research/software/Fan/Func_Linear_Model/FLM_web")

source("C:/NICHD/Research/software/Fan/Func_Linear_Model/FLM_web/FLM_fixed_model.R")
source("C:/NICHD/Research/software/Fan/Func_Linear_Model/FLM_web/FLM_FPCA.R")
source("C:/NICHD/Research/software/Fan/Func_Linear_Model/FLM_web/FLM_FPCA_no_position.R")
source("C:/NICHD/Research/software/Fan/Func_Linear_Model/FLM_web/FLM_beta_smooth_only.R")

suppressPackageStartupMessages(library(fda))
suppressPackageStartupMessages(library(MASS))
suppressPackageStartupMessages(library(getopt))
suppressPackageStartupMessages(library(SKAT))

#Read the command-line options
opt <- getopt(matrix(c('data',  'd', 1, 'character', "file with genotypes and phenotypes",
                       'range', 'r', 1, 'integer', "range of extendsion",
                       'annotation', 'a', 1, 'character', "snp annotation file",
                       'gene', 'e', 1, "character", "gene information file",
                       'output', 'o', 1, "character", "output file",
                       'seed', 's', 1, "integer", "random seed",
                       'times', 't', 1, 'integer', 'permuation times',
                       'freq', 'q',1,'double', 'max freq',
                       'format', 'f', 1,'character', 'file format',
                       'permu','p',1,'character','permutation'), byrow=T, nrow=10));

if ( is.null(opt$range ) ) { opt$range = 50000 }
if ( is.null(opt$seed  ) ) { opt$seed = 0 }
if ( is.null(opt$times ) ){opt$times=5000 }
if ( is.null(opt$freq  ) ){opt$freq=1}
if ( is.null(opt$format) ){opt$format="f"}
if ( is.null(opt$permu ) ){ opt$permu = "F" }
if ( is.null(opt$output ) ){ opt$output = "RCF_gbasis=200_order=4_bbasis=15_fbasis=25.csv" }

genotype       <- read.csv(file='C:/NICHD/Research/software/Fan/data/geno1.csv')
gene.list      <- read.csv(file='C:/NICHD/Research/software/Fan/data/gene_annotation.csv')
snp.Map        <- read.csv(file='C:/NICHD/Research/software/Fan/data/gene_map.csv') 

snpName <- as.vector( snp.Map$SNP )	   #set the snp number
snpPos  <- as.vector( snp.Map$Position )	   #set snp position
snpChr  <- as.vector( snp.Map$CHROM )

###Generate phenotype and covariate data ###
covariate <- read.csv(file='C:/NICHD/Research/software/Fan/data/cov1.csv')
y     <- read.csv(file='C:/NICHD/Research/software/Fan/data/y1.csv')
covariate <- as.matrix(covariate)
y <- y[, 1]

#Get the genotype data 
maf                = colMeans(genotype, na.rm = TRUE)/2
genotype[,maf>0.5] = 2-genotype[,maf>0.5]
maf[maf>0.5]       = 1-maf[maf>0.5]
xman               = opt$freq
genotype           = genotype[,maf<xman]

snpName = snpName[maf<xman]
snpPos  = snpPos[maf<xman]
snpChr  = snpChr[maf<xman]

geneChr <- as.vector( gene.list$Chromosome )
geneSym <- as.vector( gene.list$Gene_Symbol)
geneSta <- as.vector( gene.list$Start)
geneEnd <- as.vector( gene.list$End)

print(dim(genotype))
print(length(y))
print(length(geneChr))
set.seed(opt$seed)

order  =  4
bbasis = 15
gbasis = 15
fbasis = 25
gfasis = 25

####
flm.test.gwas<- function( mode, gene.symbol, gene.chrom, gene.start, gene.end, snp.name, snp.position, 
                          snp.chrom, pheno, pheno.symbol, covariate, genodata, rng, permutation, permu)
   {
   cat("Start calculation!\n")
   genodata = as.matrix(genodata)
   if( !is.matrix(genodata) ) 
      stop('Data cannot be convert to matrix')
   if( length(pheno) != dim(genodata)[1] ) 
      stop('The dimension of phenotype is not match with sample size')
   rlt <- data.frame() 				#initial the result
  
   for ( i in 1:dim(gene.list)[1] ) 
      {  #Test each gene in gene.csv
      sub.idx <- snp.chrom == gene.chrom[i]   	#specify the chromosome
      sub.idx <- sub.idx & (snp.position > (gene.start[i] -rng) )  #choose the SNPs from the gene start minus range set before
      sub.idx <- sub.idx & (snp.position < (gene.end[i] + rng) ) #choose the SNPs from the gene end plus range set before
    
      x <- genodata[,sub.idx] 			#get the test genotype data
      rlt[i,"Gene"] <- gene.symbol[i]	#record the tested gene symbol 

      pos <- snp.position[sub.idx]		#get the test SNP position

      if ( length(pos) >= 3 )
         {
         snpNumber = length(pos)
      
         rlt[i,"Snp_Number"] <- snpNumber			    #record how many snps is in the test.
      
         rlt[i,"Fixed_bspline$F"]   <- as.numeric(try( flm_fixed_model(pheno, mode, x, pos, order, bbasis, fbasis, gbasis, covariate, base = "bspline", interaction=FALSE)$F )) 
         rlt[i,"Fixed_bspline$LRT"] <- as.numeric(try( flm_fixed_model(pheno, mode, x, pos, order, bbasis, fbasis, gbasis, covariate, base = "bspline", interaction=FALSE)$LRT ))
         
         rlt[i,"Fixed_fspline$F"]   <- as.numeric(try( flm_fixed_model(pheno, mode, x, pos, order, bbasis, fbasis, gfasis, covariate, base = "fspline", interaction=FALSE)$F ))         
         rlt[i,"Fixed_fspline$LRT"] <- as.numeric(try( flm_fixed_model(pheno, mode, x, pos, order, bbasis, fbasis, gfasis, covariate, base = "fspline", interaction=FALSE)$LRT ))
         
         rlt[i,"beta_smooth_only_bspline$F"]   <- as.numeric(try( flm_beta_smooth_only(pheno, mode, x, pos, order, bbasis, covariate, base = "bspline", interaction=FALSE)$F )) 
         rlt[i,"beta_smooth_only_bspline$LRT"] <- as.numeric(try( flm_beta_smooth_only(pheno, mode, x, pos, order, bbasis, covariate, base = "bspline", interaction=FALSE)$LRT )) 
        
         rlt[i,"beta_smooth_only_fspline$F"]   <- as.numeric(try( flm_beta_smooth_only(pheno, mode, x, pos, order, fbasis, covariate, base = "fspline", interaction=FALSE)$F ))
         rlt[i,"beta_smooth_only_fspline$LRT"] <- as.numeric(try( flm_beta_smooth_only(pheno, mode, x, pos, order, fbasis, covariate, base = "fspline", interaction=FALSE)$LRT ))
         
         geno = x * 0
      
         for (k in 1:nrow(x))
            for (j in 1:ncol(x))
            {
            if (mode == "Dom")
               {
               if (x[k, j] == 1 || x[k, j] == 2)
                  geno[k,j] = 1
               }
            else if (mode == "Rec")
               if (x[k, j] == 2)
                  geno[k, j] = 1
            }
      
         if  ( mode == "Rec" || mode == "Dom")
            {
            x_Skat = geno
            }else if ( mode == "Additive")
               x_Skat = x   
         obj           <- SKAT_Null_Model( pheno ~ covariate, out_type = "C" )
         rlt[i,"SKAT"] <- as.numeric(try(SKAT(x_Skat, obj)$p.value))            
         }  

      print( i/dim(gene.list)[1])
      }  
   rlt
  
   File_rlt <- sprintf("C:/NICHD/Research/software/Fan/Func_Linear_Model/FLM_web/%s_mode=%s_order=4_bbasis=%d_fbasis=%d.csv", pheno.symbol, mode, bbasis, fbasis)
   write.csv(rlt, file = File_rlt, row.names = FALSE)
   }

Add <- flm.test.gwas(mode = "Additive", gene.symbol = geneSym, gene.chrom = geneChr, gene.start = geneSta, gene.end = geneEnd,
                         snp.name = snpName, snp.position = snpPos, snp.chrom = snpChr, pheno = y, pheno.symbol = "y", covariate,
                         genodata = genotype, rng = opt$range, permutation, permu = opt$times)
#Dom <- flm.test.gwas(mode = "Dom", gene.symbol = geneSym, gene.chrom = geneChr, gene.start = geneSta, gene.end = geneEnd,
#                         snp.name = snpName, snp.position = snpPos, snp.chrom = snpChr, pheno = y, pheno.symbol = "y", covariate,
#                         genodata = genotype, rng = opt$range, permutation, permu = opt$times)
#Rec <- flm.test.gwas(mode = "Rec", gene.symbol = geneSym, gene.chrom = geneChr, gene.start = geneSta, gene.end = geneEnd,
#                         snp.name = snpName, snp.position = snpPos, snp.chrom = snpChr, pheno = y, pheno.symbol = "y", covariate,
#                         genodata = genotype, rng = opt$range, permutation, permu = opt$times)


